package member

import (
	"biz-member-gateway/internal/domain"
	"net/http"

	ers "code.30cm.net/digimon/library-go/errors"

	"biz-member-gateway/internal/logic/member"
	"biz-member-gateway/internal/svc"
	"biz-member-gateway/internal/types"

	"github.com/zeromicro/go-zero/rest/httpx"
)

// 更新 Access Token
func RefreshAccessTokenHandler(svcCtx *svc.ServiceContext) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		var req types.UpdateTokenReq
		if err := httpx.Parse(r, &req); err != nil {
			e := ers.InvalidFormat(err.Error())
			httpx.WriteJsonCtx(r.Context(), w, e.HTTPStatus(), types.BaseResponse{
				Status: types.Status{
					Code:    int64(e.FullCode()),
					Message: err.Error(),
				},
			})

			return
		}

		if err := svcCtx.Validate.ValidateAll(req); err != nil {
			e := ers.InvalidFormat(err.Error())
			httpx.WriteJsonCtx(r.Context(), w, e.HTTPStatus(), types.BaseResponse{
				Status: types.Status{
					Code:    int64(e.FullCode()),
					Message: err.Error(),
				},
			})

			return
		}

		l := member.NewRefreshAccessTokenLogic(r.Context(), svcCtx)
		resp, err := l.RefreshAccessToken(&req)
		if err != nil {
			e := ers.FromError(err)
			if e.FullCode() == 0 {
				e = ers.FromGRPCError(err)
			}
			httpx.WriteJsonCtx(r.Context(), w, e.HTTPStatus(), types.BaseResponse{
				Status: types.Status{
					Code:    int64(e.FullCode()),
					Message: e.Error(),
				},
			})
		} else {
			httpx.WriteJsonCtx(r.Context(), w, http.StatusOK, types.BaseResponse{
				Status: types.Status{
					Code:    domain.SuccessCode,
					Message: domain.SuccessMessage,
				},
				Data: resp,
			})
		}
	}
}
