package member

import (
	"context"

	"code.30cm.net/digimon/app-cloudep-member-server/pkg/domain/member"
	"code.30cm.net/digimon/library-go/errs"
	memberProto "code.30cm.net/digimon/proto-all/pkg/member"

	"biz-member-gateway/internal/svc"
	"biz-member-gateway/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type PreVerifyUpdatePasswordCodeLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

// NewPreVerifyUpdatePasswordCodeLogic 預先驗證驗證碼
func NewPreVerifyUpdatePasswordCodeLogic(ctx context.Context, svcCtx *svc.ServiceContext) *PreVerifyUpdatePasswordCodeLogic {
	return &PreVerifyUpdatePasswordCodeLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *PreVerifyUpdatePasswordCodeLogic) PreVerifyUpdatePasswordCode(req *types.PreVerifyForgetPasswdReq) (resp *types.RespOK, err error) {
	// 先驗證，不刪除
	if _, err := l.svcCtx.MemberRPC.CheckRefreshCode(l.ctx, &memberProto.VerifyRefreshCodeReq{
		Account:    req.Identifier,
		CodeType:   int32(member.GenerateCodeTypeForgetPassword),
		VerifyCode: req.VerifyCode,
	}); err != nil {
		return nil, errs.Forbidden("failed to get verify code")
	}

	return
}
